--[[ Common ]]

--const
local MIN_VISIBLE_BUTTONS_SETTINGS_MODE = 1
FBC_BUTTON_PLACE_SIZE 	= 36
FBC_BUTTON_PLACE_OFFSET	= 4
FBC_FRAME_OFFSET 		= 5
FBC_ARROW_OFFSET 		= 4

--arrow button directions
FBC_DIR_UP		= 1
FBC_DIR_LEFT	= 2
FBC_DIR_DOWN	= 3
FBC_DIR_RIGHT	= 4

function FlyoutButton_GetFullSpellName(name, rank)
	if not(rank) then
		rank = ""
	end
	if (name) then
		return name.."("..rank..")", name
	end
	return nil, nil
end

function FlyoutButton_GetGenericSpellNameById(id)
	local name, rank = GetSpellInfo(id)
	return FlyoutButton_GetFullSpellName(name, rank)
end

--[[local CompanionTypes = {"MOUNT", "CRITTER"}
local SpellCacheToSlot = {}

local function CacheCompanionSpells(compType, cache2slot)
	print("GetNumCompanions", compType, GetNumCompanions(compType))
	for slot = 1, GetNumCompanions(compType) do
		--creatureID, creatureName, creatureSpellID, icon, issummoned
		local creatureID, name, id = GetCompanionInfo(compType, slot)
		print(creatureID, name, id)
		local spellName = GetSpellInfo(id)
		if spellName then
			cache2slot[spellName] = {slot, compType}
			print("CacheCompanionSpells", spellName, slot, compType)
		end
	end
end

function FlyoutButton_CacheSpells()
	SpellCacheToSlot = {}
	for _, v in pairs(CompanionTypes) do
		CacheCompanionSpells(v, SpellCacheToSlot)
	end
end

function FlyoutButton_GetCompanionInfoByName(name, compType)
	local packed = SpellCacheToSlot[name]
	if packed then
		local slot = unpack(packed)
		if slot then
			--creatureID, creatureName, creatureSpellID, icon, issummoned
			return GetCompanionInfo(compType, slot)
		end
	end
	return nil
end]]

function FlyoutButton_FindSpellId(spellName)
	local link = GetSpellLink(spellName)
	if link then
		return tonumber(select(3, strfind(link, "spell:(%d+)|")))
	else
		return nil
	end
end

function FlyoutButton_SetCursor(command, value, subValue, id)
	--print("SetCursorValues", command, value, subValue, id)
	if not(command) then
		return
	end

	ClearCursor()
	if (command == "spell") then
		PickupSpell(id)
	elseif (command == "item") then
		PickupItem(value)	--itemID or "itemString" or "itemName" or "itemLink"
	elseif (command == "macro") then
		PickupMacro(value)
	elseif (command == "equipmentset") then
		PickupEquipmentSetByName(value)
	--elseif (command == "battlepet") then
	--	C_PetJournal.PickupPet(value, PetJournal.isWild)
	end
end

local MountJournalIndexToSpellID = {}
local MountJournalUselessIndexToIndex = {}

function FlyoutButton_MountJournal_UpdateCachedList()
	for i=1, C_MountJournal.GetNumMounts() do
		local creatureName, spellID, icon, active, isUsable, sourceType, isFavorite, _, _, hideOnChar, isCollected = C_MountJournal.GetMountInfo(i)

		MountJournalIndexToSpellID[i] = spellID
		--[[if isCollected then
			print("MountJournal_UpdateCachedList", i, spellID)
		end]]
	end
end

local function HookSecureFunc_C_MountJournal_Pickup(Index)
	local _, UselessIndex = GetCursorInfo()
	if (Index and UselessIndex) then
		MountJournalUselessIndexToIndex[UselessIndex] = MountJournalIndexToSpellID[Index]
		--print("Index", "UselessIndex", Index, UselessIndex)
	end
end
hooksecurefunc(C_MountJournal, "Pickup", HookSecureFunc_C_MountJournal_Pickup)

function FlyoutButton_GetCursorValues()
	local command, value, subValue, id = GetCursorInfo()
	--print("GetCursorValues", command, value, subValue, id)
	if (command == "spell") then
		value = FlyoutButton_GetGenericSpellNameById(id)
	elseif (command == "item") or (command == "equipmentset") then
		--nothing to do, value is item id
	elseif (command == "macro") then		
		subValue = value -- value is macro index	
		value = GetMacroInfo(value) --name, texture, body
	elseif (command == "mount") then
		command = "spell"
		id = MountJournalUselessIndexToIndex[value]
		value = FlyoutButton_GetGenericSpellNameById(id)
		subValue = "spell"
		--print(command, value, subValue, id)		
	--[[elseif (command == "companion") then
		local _
		_, _, id = GetCompanionInfo(subValue, value)
		--without rank, else can't find texture by name
		value = GetSpellInfo(id)
		command = "spell"
		print("GetCursorValues 'companion'", command, value, subValue, id)]]
	end
	return command, value, subValue, id
end

function FlyoutButton_GetListButtonsCount(listTable)
	local count = 0
	if listTable then
		for i = 1, #listTable do
			if (listTable[i]) and (listTable[i].value) then
				count = i
			end
		end
	end
	local settingsCount = count
	
	if FbcSettingsMode then
		if count < MIN_VISIBLE_BUTTONS_SETTINGS_MODE then
			settingsCount = MIN_VISIBLE_BUTTONS_SETTINGS_MODE
		else
			settingsCount = count + 1
		end
	end
	return count, settingsCount
end
